package Xray::FITS;
require Exporter;

use strict;
use warnings;

use Xray::Tools;

our $VERSION = 1.00;

our @ISA = ("Exporter");
our @EXPORT = qw(
 CreateLc
 FitsDump 
 ReadKey
 StartTime
);

################################################################

sub CreateLc
{
  my ($outfile, $time, $rate, $error) = @_;
  
  my $tstart = $time->[0];
  my $tstop = $time->[scalar @$time - 1];
  my $timedel = $time->[1] - $time->[0];
  
  my $CDFILE = "col.txt$$";
  open(CDFILE,">$CDFILE");
  print CDFILE <<EOC;
TIME  1D   s
RATE 1E count/s
ERROR 1E count/s
EOC
  close(CDFILE);

  my $HEADFILE = "header.txt$$";
  open(HEADFILE, ">$HEADFILE");
  print HEADFILE <<EOH;
TIMEUNIT 's'
TSTART $tstart
TSTOP $tstop
TIMEDEL $timedel
TIMEZERO 0
HISTORY File created by CreateLc
EOH
  close(HEADFILE);

  my $DATAFILE = "data.txt$$";
  open(DATAFILE, ">$DATAFILE");
  for my $i (0 .. scalar(@$rate) - 1)
  {
    my $rec = $time->[$i] . ' ' . $rate->[$i] . ' ' . $error->[$i] . "\n";
    print DATAFILE $rec;
  }
  close(DATAFILE);

  my $stat;
  my $args = "fcreate cdfile=$CDFILE datafile=$DATAFILE outfile=$outfile headfile=$HEADFILE tbltype=\"binary\" nskip=0 nrows=0 history=no morehdr=0 extname=\" \" anull=\" \" inull=0 clobber=yes";
  my @log = &call($args, $stat);
  die "Problems creating light curve\n @log\n" if($stat);
  unlink($CDFILE, $HEADFILE, $DATAFILE);
}

#######################################################

sub FitsDump 
#
#  Dump a FITS file into an array
#
#    @fits = FitsDump($infile, $columns);
#
# INPUT:
#
#  $infile - input file (with extension, perhaps)
#  $colums - list of columns, e.g. $columns =  "time rate error fracexp"
#
# OUTPUT:
#
#  @fits - array (columns) of arrays (raws)
#
{
  my  $infile = $_[0];
  my  $columns = $_[1];
  my (@s, @a, $line);
  my @fits = ();

  @s = split('\s*\n', `fdump prhead=no infile=${infile} outfile=STDOUT columns=\"${columns}\" showcol=no showrow=no showunit=no rows=- more=yes wrap=yes pagewidth=200`);
  foreach $line (@s)
  {
    if($line =~ /\S/)
    {
      @a = split ' ', $line;
      push @fits, [@a];
    }
  }
  return @fits;
}

################################################################

sub ReadKey
#
# Reads a key from FITS file
#
# $key_value = ReadKey($file, $keyname);
#
# e.g. my $exposure = ReadKey($file, 'EXPOSURE');
#
{
  my ($file, $keynam) = @_;
  
  my @key = grep /=/, `fkeyprint infile=${file}+1 keynam=${keynam} outfile=STDOUT`;
  $key[0] =~ s/\'//g;
  $key[0] =~ /=\s+(.*)\s+\//;
  return $1;
}

################################################################

sub StartTime
#
# Read TSTART keyword from the list of files and put it into hash table
#
#   %timefile = StartTime(\@file_list);
#
# INPUT
#
#   file_list - list of input files
#
# OUTPUT:
#
#   timefile - hash table (tstart -> file_name)
#
{
  my $file_list = $_[0];

  my %timefile = ();
  my ($tstart);

  foreach my $file (@$file_list)
  {
    system "fkeypar $file TSTART";  # read TSTART keyword
    $tstart = `pget fkeypar value`;
    chop($tstart);
    $timefile{$tstart} = $file;
    print "$file -> $tstart\n";
  }
  return %timefile;
}
#############################################################################



1;
