package Xray::Sigma;
require Exporter;

use strict;
use warnings;

our $VERSION = 1.00;

our @ISA = ("Exporter");
our @EXPORT = qw(AbsSigma);


####################################################

sub AbsSigma
#
# Cross section (in sigma_T units) for neutral absorption. 
# Morrison & McCammon 1983, ApJ, 270, 119
#
# $a = AbsSigma($x)
#
#   $x - energy in keV
#   $a - sigma_abs / sigma_T
#
{
  my $x = shift;
  
  my @c0 = (0.336, 0.173, 0.346, 0.781, 0.714, 0.955, 3.089, 1.206, 1.413, 2.027, 3.427, 3.522, 4.339, 6.29, 7.012, 9.532);
  my @c1 = (0.0, 6.081, 2.679, .188, .668, 1.458, -3.806, 1.693, 1.468, 1.047, 0.187, 0.187, -0.024, 0.309, 0.252, 0.0);
  my @c2 = (0.0, -21.5, -4.761, 0.043, -0.514, -0.611, 2.94, -0.477, -0.315, -0.17, 0.0, 0.0, 0.0075, 0.0, 0.0, 0.0);
  my @er = (0.03, 0.1, 0.284, 0.4, 0.532, 0.707, 0.867, 1.303, 1.840, 2.471, 3.210, 4.038, 7.111, 8.331, 10.0, 1.0e10);
  
  my $sigmaT = 6.6524615e-3;  # sigma_T in units of 10^-22 cm^2
  
  if($x <= 0.03) 
    {return 0.336 / ($sigmaT * $x*$x*$x);}
  if($x >= 10.0)
    {return 9.532 / ($sigmaT * $x*$x*$x);}

  my $mid;
  my $left = 0;
  my $right = @er;
  while(abs($right - $left) > 1)
  {
    $mid = int ($left + $right) / 2;
    if(($er[$left] - $x) * ($er[$mid] - $x) > 0)
      {$left = $mid;}
    else
      {$right = $mid;}
  }
  if($er[$mid] > $x)
    {$mid--;}
  
  $mid++;
  my $sig = ($c0[$mid] + $c1[$mid]*$x + $c2[$mid]*$x*$x) / ($x*$x*$x);
  return $sig / $sigmaT;
}



1;

