package Xray::Tools;
require Exporter;

use Term::Complete;
use File::Copy;
use PDL;
use PDL::NiceSlice;

use strict;
use warnings;

our $VERSION = 1.00;

our @ISA = ("Exporter");
our @EXPORT = qw(
 call warn ToBe
 Percent
 Percent2
 ReadASCII
 ReadFileList
 ReadLine
 ReadName
 ReadYesNo
 Retrieve
 rmatrix
 Rmmin
 Store
 WriteASCII
);

################################################################

sub call
#
# Call to a system task
#
# @log = call($arguments, $status [, $ignore])
#
# Call a system command (e.g. an ftool) and process errors. Returns $status = $?,
# i.e. 0 if there is no error.
#
# If $ignore == 1 then warnings are ignored (i.e. $? = 0, but there is something
# sent to STDERR)
#
{
  my $command = $_[0];
  my $ignore = (defined $_[2]) ? $_[2] : 0;
  
  
  my $errfile = "/tmp/error$$";
  unlink $errfile;
  
  my @result = `$command 2> $errfile`;
  
  $_[1] = $?;
  if($?)
  {
    print "\n### This command went wrong:\n";
    print "$command\n\n";
  }
  my $errmsg = '';
  if(open ERR, $errfile)
  {
    while(my $line = <ERR>)
    {
      chomp $line;
      $errmsg .= "$line\n" if $line !~ /^\s*$/;
    }
  }
  if($errmsg && !($ignore && !$?))
  {
    print "\n### Error output:\n";
    print "$errmsg\n";
  }
  unlink $errfile;
  return @result;
}

sub call_old {

# @log = &call($args,"warn","Problem running:\n $args","logfile.log")

## based on runcom, but keeps the output from the ftool even in the case
## of an error;  give command, error routine, error string, error log;
## returns array:  1st value is 0 if good or $? or 1 if system or ftool error; 
## then command; then error messages sent by ftool or system to 
## STDERR; then any other output from tool;  retains suppress option.

    my($command) = $_[0];
    my($ERRFIL,@result,$CURHANDLE,$errormsg);
    my $suppress;
    
    if($command =~ s/^-// ) {$suppress = 1;}
    
    $ERRFIL = "/tmp/error$$";
    unlink $ERRFIL;
    $command .= " 2> $ERRFIL";
    @result = `$command`;

    if ( !open(ERRFIL, $ERRFIL) ) {  ## if no ERRFIL created, return with $? status
        unlink $ERRFIL;
    }
    else {
LOP:  while($_ = <ERRFIL>)
             {
	       last LOP if !(/(ld\.so|^\s*\n$)/);
             }
      if ( ! $_ ) {  ## if ERRFIL created but empty, return
	  unlink $ERRFIL;
      }
      else {  ##  there is an error message
	  unless ($suppress) {
	    $errormsg = $_;
            unlink $ERRFIL;
            }
      }  ## else there's an error message
    } ##  else there's an ERRFIL created

    if ($? || ($errormsg))        ## call err_routine if either $? or $errormsg
    { 
      $CURHANDLE = select(STDERR);
      print "\n-------------------------------------------------------------\n";
      print "Something went wrong:\n";
      print "$command\n";
      print "-------------------------------------------------------------\n";
      select($CURHANDLE);

      if($errormsg)
      {
        push @result, "Problem running:\n $command:\n\n$errormsg\n";
        $_[1] = 1;
      }
      else
      {
        push @result, "Problem running:\n $command\n";
        $_[1] = $?;
      }
    } ## if any errors

    else 
    {
      push @result, "$command\n";
      $_[1] = $?;
    }
    return @result;

} ## sub call

sub warn {

    my $string = $_[0];
    my $log = $_[1];
    my $CURHANDLE = select(STDERR);
    print "-------------------------------------------------------------\n";
    print "$string\n";
    print "See full log in $log\n" if $log;
    print "-------------------------------------------------------------\n";
    select($CURHANDLE);
}

sub ToBe
{
  my $stat = $_[0];
  my $log = $_[1];

  if($stat)
  {
    die("I'm dead.\nCheck the log file $log.\n\n");
  }
}

###########################################################################

sub Percent
#
# Print percentage
#
{
  my $p = 100 * $_[0];

  printf "\b\b\b\b%3.0f%%", $p;
}

sub Percent2
{
  my $p = 100 * $_[0];

  printf "\b\b\b\b\b\b%5.2f%%", $p;
}

#######################################################

sub ReadASCII
#
# Reads columns from an ASCII file into an array
#
# @arr = ReadASCII($filename)
#
{
  my $file = shift;
  local *IN;
  open(IN, "$file") or die "Arrrgh: $!\n";
  my @in = ();
  while(<IN>)
  {
    my @arr = split;
    push @in, [@arr];
  }
  close(IN);
  return @in;
}


################################################

sub ReadFileList
#
# Read list of pathnames from an ASCII file
#
#   @list = ReadFileList($file_name);
#
# INPUT:
#
#   file_name - name of a file, containing list of pathnames
#
# OUTPUT:
#
#   list - list of pathnames
#
{
  my $file_name = $_[0];
  my @file_list = ();
  my $line;

  local *DATA;
  open(DATA, $file_name) 
    or die("\n*** Cannot open input file $file_name.\n\n");
  while($line = <DATA>) 
  {
    chomp $line;
    push(@file_list, $line) if $line !~ /^\s*$/;
  }
  close(DATA);
  return @file_list;
}

################################################


sub ReadLine
#
# Reads a line of text into an array, separator is space
#
#    @arr = ReadLine($line);
#
# INPUT:
#
#   line - a line of text
#
# OUTPUT
#
#   array - items from the line
#
{
  my $line = $_[0];
  my @arr = ();

  chop $line;
  $line =~ /( *)(.*)/;
  @arr = split(/ +/, $2);
  return @arr;
}


################################################################

sub ReadName
#
# Read a name from standard input
#
#   $answer = ReadName("$query", "$defaut");
#
# INPUT:
#
#   query - a question asked
#   default - default answer
#
# OUTPUT:
#
#   answer - answer typed; if only Enter hit, <default> is returned
#
# Queries and returns the answer. If return is typed, the default answer 
# is returned.
#
{
  my ($query, $default_answer) = @_;
  local *DIR;

  opendir(DIR, '.') or die $!;
  my @d = readdir(DIR);
  my $answer = Complete("$query [$default_answer]: ", \@d);
  if($answer eq "") {$answer = $default_answer;}
  if($answer eq "kill") {die "\nYou killed me, I'm dead, good bye.\n";}
  return $answer;
}

################################################

sub ReadYesNo
#
# Read a yes-or-no answer from standard input
#
#   $answer = ReadYesNo("$query", $defaut);
#
# INPUT:
#
#   query - a question asked
#   default - default answer (0/1)
#
# OUTPUT:
#
#   answer - 0 or 1
#
#
{
  my ($query, $default_answer) = @_;

  my $answer;
  my $def = ($default_answer) ? "y" : "n";
  do{
    print "$query (y/n) [$def]:";
    $answer = <STDIN>;
    chomp $answer;
  } until ($answer eq "" || $answer =~ /^[yn]$/);
  if($answer eq "") {$answer = $default_answer;}
#  if($answer eq "kill") {die "\nYou killed me, I'm dead, good bye.\n";}
  if($answer eq "y") {$answer = 1;}
  if($answer eq "n") {$answer = 0;}
  return $answer;
}

################################################################

sub Retrieve
#
# Retrieves parameters from the file
#
#  Retrieve($filename, $par1, $par2, ..., $parn);
#
# INPUT:
#
# $filname - name of the file (e.g. ".rc")
#
# OUTPUT
#
# $par<i> - paramter
#
{
  my $file_name = shift(@_);
  my ($par);

  if(-e $file_name)
  {
    local *RC;
    open(RC, $file_name) or die("Cannot open resource file $file_name.\n");
    my $i = 0;
    while($par = <RC>)
    {
      chop($par);
      $_[$i] = $par;
      $i++;
    }
  }
}

################################################################

sub rmatrix
#
# Like rcols, but reads two-dimentional matrix into piddle
#
# $pdl = rmatrix $file;
#
{
  my $file = shift;
  
  local *IN;
  open(IN, $file) or die "$0: Cannot open file $file\n";
  my @in = <IN>;
  close(IN);
  my $ny = @in;
  my @row = split " ", $in[0];
  my $nx = @row;
  my $img = zeroes($nx, $ny);
  my $i = 0;
  for my $row (@in)
  {
    chomp $row;
    $img(0:-1, $i++) .= pdl(split " ", $row);
  }
  return $img;
}

################################################################

sub Rmmin
#
# Removes minus signs breaking lines in QDP output from XSPEC.
# Lines are joined together.
#
{
  my $filename = shift;
  local (*IN, *OUT);
    
  my $tmpfile = "${filename}~";
  move($filename, $tmpfile) or die "$!\n";

  open(IN, "$tmpfile") or die "$!\n";
  open(OUT, ">$filename") or die "$!\n";
  while(my $line = <IN>)
  {
    chomp($line);
    if($line =~ /(.*)(-$)/)
      {print OUT "$1";}
    else
      {print OUT "$line\n";}
  }
  close(IN);
  close(OUT);
}

################################################################

sub Store
#
# Stores parameters in a file
#
#  Store($filename, $par1, $par2, ..., $parn);
#
# INPUT:
#
# $filname - name of the file (e.g. ".rc")
# $par<i> - paramter
#
{
  my $file_name = shift;

  local *RC;
  open(RC, ">$file_name") or die("Cannot open recource file $file_name.\n");
  foreach my $par (@_)
  {
    print RC "$par\n";
  }
  close(RC);
}

################################################################

sub WriteASCII
#
# Writes columns from an array into ASCII file
#
# WriteASCII(\@arr, $filename)
#
{
  my $data = $_[0];
  my $filename = $_[1];
  local *TMP;
  
  open(TMP, ">$filename") or die "$!\n";;
  foreach my $bin (@$data)
  {
    print TMP "@$bin\n";
  }
  close(TMP);
}



1;
